﻿using System;
using System.Text;
using System.Collections.Generic;

namespace JSonSilverlight.JSonWrapper
{
    internal static class HelperInternal
    {
        static Type[] ValidTypes = new Type[] {typeof(JsonArray),typeof(JsonObject),
                        typeof(string),typeof(bool),typeof(byte),typeof(sbyte),
                        typeof(short),typeof(ushort),typeof(int),typeof(uint),typeof(long),typeof(ulong),
                        typeof(float),typeof(double),typeof(decimal)};

        internal static void CheckValidType(object Value)
        {
            if (Value != null)
            {
                for (int i = 0; i < ValidTypes.Length; i++) if (Value.GetType() == ValidTypes[i]) return;
                throw new FormatException("Invalid value type: " + Value.GetType().ToString());
            }
        }
        internal static void WriteString(StringBuilder sb, string s)
        {
            sb.Append('"');
            for (int i = 0; i < s.Length; i++)
            {
                char c = s[i];
                if (c == '"')
                {
                    sb.Append("\\\"");
                    continue;
                }
                if (c == '\\')
                {
                    sb.Append("\\\\");
                    continue;
                }
                if (c == '\n')
                {
                    sb.Append("\\n");
                    continue;
                }
                if (c == '\r')
                {
                    sb.Append("\\r");
                    continue;
                }
                if (c == '\t')
                {
                    sb.Append("\\t");
                    continue;
                }
                sb.Append(c);
            }
            sb.Append('"');
        }
    }

    public static class Helper
    {
        /// <summary>
        /// Method extracts a value (inputString) from the JSon Object jOb and returns a string.
        /// Will throw an exception if there was a problem with the extraction.
        /// </summary>
        /// <param name="inputString"></param>
        /// <param name="jOb"></param>
        /// <returns></returns>
        public static string GetStringValue(string inputString, JsonObject jOb)
        {
            string outputString = string.Empty;

            try
            {
                outputString = jOb[inputString] as string;
            }
            catch
            {
                throw new Exception("JSon Parser Exception - Failed to find the Key requested");
            }

            return outputString;
        }

        /// <summary>
        /// Method extracts a value (inputString) from the JSon Object jOb and returns an integer
        /// Will throw an exception if it fails to find the inputString value in jOb.
        /// </summary>
        /// <param name="inputString"></param>
        /// <param name="jOb"></param>
        /// <returns></returns>
        public static int GetIntegerValue(string inputString, JsonObject jOb)
        {
            int outputInteger = -1;

            try
            {
                string tmpString = jOb[inputString] as string;

                if (string.IsNullOrEmpty(tmpString))
                {
                    Int32.TryParse(tmpString, out outputInteger);
                }
            }
            catch
            {
                throw new Exception("JSon Parser Exception - Failed to find the Key requested");
            }

            return outputInteger;
        }


        /// <summary>
        /// Finds the first instance of a section in the JSon structure that matches the 'section' parameter.
        /// </summary>
        /// <param name="rootJsonObject"></param>
        /// <param name="section"></param>
        /// <returns></returns>
        public static JsonObject FindJSonSectionObject(JsonObject rootJsonObject, string section)
        {
            JsonObject aSection = null;

            if (!string.IsNullOrEmpty(section))
            {
                string tmpLower = section.ToLower();

                foreach (KeyValuePair<String, object> entry in rootJsonObject.Entries)
                {
                    string testString = entry.Key.ToLower();

                    if (testString == tmpLower)
                    {
                        aSection = (JsonObject)rootJsonObject;
                        break;
                    }
                    else
                    {
                        JsonObject aJsonObj = entry.Value as JsonObject;

                        if (aJsonObj != null)
                        {
                            aSection = FindJSonSectionObject(aJsonObj, section);

                            if (aSection != null)
                                break;
                        }
                        else
                        {
                            JsonArray ja = entry.Value as JsonArray;

                            if (ja != null)
                            {
                                foreach (var item in ja)
                                {
                                    if (item is JsonObject)
                                    {
                                        JsonObject theJObjv = item as JsonObject;

                                        if (theJObjv != null)
                                        {
                                            aSection = FindJSonSectionObject(theJObjv, section);

                                            if (aSection != null)
                                                break;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            return aSection;
        }


        /// <summary>
        /// Finds the first instance of a section in the JSon structure that matches the 'section' parameter.
        /// </summary>
        /// <param name="rootJsonObject"></param>
        /// <param name="section"></param>
        /// <returns></returns>
        public static JsonArray FindJSonSectionArray(JsonObject rootJsonObject, string section)
        {
            JsonArray aSection = null;

            if (!string.IsNullOrEmpty(section))
            {
                string tmpLower = section.ToLower();

                foreach (KeyValuePair<String, object> entry in rootJsonObject.Entries)
                {
                    string testString = entry.Key.ToLower();

                    if (testString == tmpLower)
                    {
                        aSection = entry.Value as JsonArray;
                        break;
                    }
                    else
                    {
                        JsonObject aJsonObj = entry.Value as JsonObject;

                        if (aJsonObj != null)
                        {
                            aSection = FindJSonSectionArray(aJsonObj, section);

                            if (aSection != null)
                                break;
                        }
                        else
                        {
                            JsonArray ja = entry.Value as JsonArray;

                            if (ja != null)
                            {
                                foreach (var item in ja)
                                {
                                    if (item is JsonObject)
                                    {
                                        JsonObject theJObjv = item as JsonObject;

                                        if (theJObjv != null)
                                        {
                                            aSection = FindJSonSectionArray(theJObjv, section);

                                            if (aSection != null)
                                                break;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            return aSection;
        }
    }
}
